/******************************************************************************\
**
**     COMPANY:	LDW Software
**
**      MODULE:	Achievement
**
** DESCRIPTION:	Maintains completion status of achievements
**
**
\******************************************************************************/

#ifndef ACHIEVEMENT_H
#define ACHIEVEMENT_H

#include "ldwTypes.h"
/******************************************************************************\
** LIBRARY INCLUDES
\******************************************************************************/

/******************************************************************************\
** CONSTANT DEFINITIONS
\******************************************************************************/

/******************************************************************************\
** CLASS ENUMERATIONS
\******************************************************************************/
enum EAchievement
{
	eAchievement_None  = -1,
	
	//carla new vv4 trophies
	eAchievement_LiteSnack = 0,
	eAchievement_BalancedMeal,
	eAchievement_Supersize,
	eAchievement_AllYouCanEat,
	eAchievement_Progress,
	eAchievement_ProlificProgress,
	eAchievement_AbundantProgress,
	eAchievement_ResearchDev,
	eAchievement_Shrooms,
	eAchievement_MoreShrooms,
	eAchievement_FungusAmongUs,
	eAchievement_GourmetShrooms,
	eAchievement_LoveShrooms,
	eAchievement_ObsessedShrooms,
	eAchievement_RelicCollected,
	eAchievement_ScienceItemCollected,
	eAchievement_MasterCollector,
	eAchievement_CutsBothWays,
	eAchievement_SecondThought,
	eAchievement_HeathenPrerogative,
	eAchievement_CantBeatEm,
	eAchievement_EternalDevotion,
	eAchievement_YayButterflies,
	eAchievement_Jenga,
	eAchievement_WayTooMuchSugar,
	eAchievement_TinyBlessing,
	eAchievement_TwoTinyBlessing,
	eAchievement_Help,
	eAchievement_SugarSpice,
	eAchievement_SnipsSnails,
	eAchievement_GoodStart,
	eAchievement_CrowdControl,
	eAchievement_MasterScience,
	eAchievement_MasterConstruction,
	eAchievement_MasterParenting,
	eAchievement_MasterMedicine,
	eAchievement_MasterFarming,
	eAchievement_MasterDevotion,
	eAchievement_JackTrades,
	eAchievement_SkilledWorkers,
	eAchievement_FullResearchLearning,
	eAchievement_FullResearchScience,
	eAchievement_FullResearchFood,
	eAchievement_FullResearchSpirituality,
	eAchievement_FullResearchMedicine,
	eAchievement_FullResearchConstruction,
	eAchievement_FullResearchAllTech,

	//arthurs
	eAchievement_EventfulIsland,
	eAchievement_NeverADullMoment,
	eAchievement_RoomGrow,
	eAchievement_PlacePlayLearn,
	eAchievement_NotFeelHowLook,
//	eAchievement_SwineFlu,
	eAchievement_Triage,
	eAchievement_MedicineMan,
	eAchievement_MiracleWorker,
	eAchievement_RubSticks,
	eAchievement_VigilentCamper,
	eAchievement_DidntStartFire,
	eAchievement_Fashionista,
	eAchievement_RomperRoom,
	eAchievement_ProlificProcreation,
	eAchievement_PoolParty,
	eAchievement_Wise,
	eAchievement_Elder,
	eAchievement_OverAchiever,
	eAchievement_Unachievable,

	eAchievement_NumberOf
};

const int cAchievementHeight = 60;

/******************************************************************************\
** CLASS DEFINITIONS
\******************************************************************************/

class ldwTiledBackground;

class CAchievement
{
private:

	//class structures
	struct SAchievement
	{
		bool			m_bIsComplete;
		int				m_iProgress;
		unsigned int	m_uiCompletionTime;
	};

public:

	struct SSaveState
	{
		SAchievement	m_sAchievement[eAchievement_NumberOf];
	};

public:

	CAchievement(void);
	~CAchievement(void);

	void       Reset();

	void       ResetSpecificAchievement(EAchievement achievement);

	const bool LoadState(SSaveState &sSaveState);
	const bool SaveState(SSaveState &sSaveState);

	// process
	
	// access

	void    SetComplete(EAchievement achievement);
	bool 	IsComplete(EAchievement achievement);
	int  	CompletionTime(EAchievement achievement);
	void	ResetSingleAchievementProgress(EAchievement achievement);

	void    IncrementProgress(EAchievement achievement,int increment = 1);
	int		AchievementsComplete();

	void 	DrawAchievement(int x,int y,int achievementIndex,bool forPopup = false,float alpha = 1.0f);
	void 	DrawAchievementPopup(bool reschedule);

	bool	Showing() {return m_isShowing;}

private:
	void 		  QueueAchievementNotify(EAchievement achievement);
	EAchievement  PopAchievementNotify();
	void		  ResetNotifyQueue();

	int			  PercentCompleted(EAchievement achievement);

	SAchievement	m_sAchievement[eAchievement_NumberOf];
	EAchievement	m_acheivementNotifyQueue[eAchievement_NumberOf];

	ldwUINT32	  	m_notifyTimer;
	bool			m_isShowing;

	ldwTiledBackground *mBackground;
};

/******************************************************************************\
** GLOBAL DECLARATIONS
\******************************************************************************/
extern CAchievement		Achievement;

#endif

/******************************************************************************\
** END OF FILE
\******************************************************************************/