/******************************************************************************\
**
**     COMPANY:	LDW Software
**
**      MODULE:	AnimManager
**
** DESCRIPTION:	Manages a set of animation sequences
**
**
\******************************************************************************/

#ifndef ANIM_MANAGER_H
#define ANIM_MANAGER_H

/******************************************************************************\
** LIBRARY INCLUDES
\******************************************************************************/

#define HAVE_HIRES_DETAIL  1
#define HAVE_HEATHEN_MASKS 1


#include "BodyPositions.h"

#include "../Game/Isola.h"	//TODO: EDirection - move here?
#include "../Utils/theGraphicsManager.h"
#include "../Villagers/VillagerBio.h"

class ldwImageGrid;

/******************************************************************************\
** CONSTANT DEFINITIONS
\******************************************************************************/
const int AnimManager_c_iMaxFrames  = 16;

const int cAnimMaxNameLength = (32 - 1);

/******************************************************************************\
** CLASS ENUMERATIONS
\******************************************************************************/
enum EAnim
{
	eAnim_None = -1,

	eAnim_Walk,
	eAnim_WalkSE,		// for Alignment/Editor
	eAnim_WalkSW,		// for Alignment/Editor
	eAnim_WalkNW,	    // for Alignment/Editor

	eAnim_Bend,
	eAnim_Cheer,
	eAnim_Climb,
	eAnim_Dance,
	eAnim_Dive,
	eAnim_Embrace,
	eAnim_Exorcist,
	eAnim_Joy_Twirl_CW,
	eAnim_Jump,
	eAnim_Twirl_CCW,
	eAnim_Twirl_CW,
	eAnim_Disco,
	eAnim_DoTheMonkey,
	eAnim_Revel,
	eAnim_LieDown,
	eAnim_Scrub,
	eAnim_ShakeHead,
	eAnim_SwimOnBack,
	eAnim_Work,
	eAnim_Pushups,

/*
	eAnim_SitNE,
	eAnim_SitNW,
	eAnim_InChairNE,
	eAnim_InChairNW,
	eAnim_LieSW,
*/
	eAnim_NumberOf
};

const int cAnimMaxAnimations = eAnim_NumberOf + 100;

enum EAnimEffect
{
	eAnimEffect_OneShot,
	eAnimEffect_Loop,
	eAnimEffect_Bounce
};

enum EAnimFrame
{
	eAnimFrame_None = -1,
	eAnimFrame_RandomWalk = -2,

	eAnimFrame_Walk1 = 0,
	eAnimFrame_Walk2,
	eAnimFrame_Walk3,
	eAnimFrame_Walk4,
	eAnimFrame_Walk5,
	eAnimFrame_Walk6,
	eAnimFrame_Walk7,
	eAnimFrame_Walk8,

	eAnimFrame_WalkNE1 = eAnimFrame_Walk1,
	eAnimFrame_WalkNE2 = eAnimFrame_Walk2,
	eAnimFrame_WalkNE3 = eAnimFrame_Walk3,
	eAnimFrame_WalkNE4 = eAnimFrame_Walk4,
	eAnimFrame_WalkNE5 = eAnimFrame_Walk5,
	eAnimFrame_WalkNE6 = eAnimFrame_Walk6,
	eAnimFrame_WalkNE7 = eAnimFrame_Walk7,
	eAnimFrame_WalkNE8 = eAnimFrame_Walk8,

	eAnimFrame_WalkSE1,
	eAnimFrame_WalkSE2,
	eAnimFrame_WalkSE3,
	eAnimFrame_WalkSE4,
	eAnimFrame_WalkSE5,
	eAnimFrame_WalkSE6,
	eAnimFrame_WalkSE7,
	eAnimFrame_WalkSE8,

	eAnimFrame_WalkSW1,
	eAnimFrame_WalkSW2,
	eAnimFrame_WalkSW3,
	eAnimFrame_WalkSW4,
	eAnimFrame_WalkSW5,
	eAnimFrame_WalkSW6,
	eAnimFrame_WalkSW7,
	eAnimFrame_WalkSW8,

	eAnimFrame_WalkNW1,
	eAnimFrame_WalkNW2,
	eAnimFrame_WalkNW3,
	eAnimFrame_WalkNW4,
	eAnimFrame_WalkNW5,
	eAnimFrame_WalkNW6,
	eAnimFrame_WalkNW7,
	eAnimFrame_WalkNW8,

	eAnimFrame_ActionStart,
	eAnimFrame_Bend1 = eAnimFrame_ActionStart,
	eAnimFrame_Bend2,
	eAnimFrame_Bend3,
	eAnimFrame_Bend4,
	eAnimFrame_Work1,
	eAnimFrame_Work2,
	eAnimFrame_Work3,
	eAnimFrame_Work4,
	eAnimFrame_Sit,
	eAnimFrame_Lie,
	eAnimFrame_PointNE,
	eAnimFrame_PointSE,
	eAnimFrame_PointSW,
	eAnimFrame_PointNW,
	eAnimFrame_ReachS,
	eAnimFrame_Wave1 = eAnimFrame_ReachS,
	eAnimFrame_ReachN,
	eAnimFrame_Wave2 = eAnimFrame_ReachN,
	eAnimFrame_Watch,

/*
	eAnimFrame_SitLieStart,
	eAnimFrame_SitNE = eAnimFrame_SitLieStart,			 
	eAnimFrame_Sit,
	eAnimFrame_SitNW,			 
	eAnimFrame_InChairArmUpNE, 
	eAnimFrame_InChairNE,		 
	eAnimFrame_InChairArmUpNW, 
	eAnimFrame_InChairNW,		 
	eAnimFrame_Lie,
	eAnimFrame_LieSW,		 
*/
	eAnimFrame_DetailStart,
	eAnimFrame_Detail = eAnimFrame_DetailStart,

	eAnimFrame_NumberOf
};

enum EHeadDirection
{
	eHeadDirection_None = -1,
	eHeadDirection_Random = -2,
	eHeadDirection_RandomUpward = -3,

	eHeadDirection_Northeast = 0,
	eHeadDirection_Southeast,
	eHeadDirection_Southwest,
	eHeadDirection_Northwest,
	
	eHeadDirection_SouthSoutheast,
	eHeadDirection_South,
	eHeadDirection_SouthSouthwest,

	eHeadDirection_Up,
/*
	eHeadDirection_UpNE1 = eHeadDirection_Up,
	eHeadDirection_UpNE2,
	eHeadDirection_UpNW1,
	eHeadDirection_UpNW2,
	eHeadDirection_Down,
*/
	eHeadDirection_NumberOf,

	eHeadDirection_Detail1 = eHeadDirection_NumberOf,
	eHeadDirection_Detail2,
	eHeadDirection_Detail3,

	eHeadDirection_NumberOfWithDetail,

	eHeadDirection_DetailSouth = 1
};

enum EAnimGender
{
	eAnimMale,eAnimFemale,eAnimGenderCount
};

enum EAnimPart
{
	eAnimHeadPart = -1,
	eAnimBodyPart1,
	eAnimBodyPart2,
	eAnimMaxBodyParts,
	eAnimSetCount = numVillagerSets,
//	eAnimMaskSetCount = 3
	eAnimMaskSetCount = 5
};

enum EAnimImage
{
	eAnimHeads,eAnimWalkingBodies,eAnimActionBodies,

	eAnimImage_NumberOf
};

/******************************************************************************\
** CLASS DEFINITIONS
\******************************************************************************/
class CAnimManager
{
	friend class theAlignVillagerScene;

private:

	struct SAnimImageDef
	{
		EImage		  imageId;
		int           imageFrame;
		ldwPoint      linkToPrev[eAnimSetCount];
		ldwPoint      linkToNext[eAnimSetCount];
	};

	struct SAnimMaskImageDef 
	{
		EImage		  imageId;
		int           imageFrame;
		ldwPoint      linkToPrev[eAnimMaskSetCount];
		ldwPoint      linkToNext[eAnimMaskSetCount];
	};
	//class structures
	struct SFrameDef
	{
		int				imageCount;
		SAnimImageDef	animImage[eAnimGenderCount][eAnimMaxBodyParts];
	};

	struct SFrame
	{
		EAnimFrame		m_eAnimFrame;
		EHeadDirection	m_eHeadDirection;
		EDirection		m_eDirection;
	};

	struct SAnim
	{
		char			m_Name[cAnimMaxNameLength + 1];
		SFrame			m_sFrame[AnimManager_c_iMaxFrames];
		int				m_iNumFrames;

		float			m_fSpeed;
		EAnimEffect		m_eAnimEffect;

		//structure methods
		//process
		public:		void Initialize(const char *name,const EAnimEffect eAnimEffect, const float fSpeed = 1.0f);
		public:		void AddFrame(const EAnimFrame eAnimFrame, const EHeadDirection eHeadDirection, const EDirection eDirection);
	};

	//class data
	SAnim		  m_sAnim[cAnimMaxAnimations];

	int			  m_AnimCount;

	SFrameDef         m_sFrameDef[eAnimFrame_NumberOf];
	SAnimImageDef     m_sHeadImageDef[eAnimGenderCount][eHeadDirection_NumberOfWithDetail];
#ifdef HAVE_HEATHEN_MASKS
	SAnimMaskImageDef m_sMaskImageDef[eHeadDirection_NumberOf];
#endif

#if _DEBUGxx
	SAnimImageDef m_sOldHeadImageDef[eAnimGenderCount][eHeadDirection_NumberOf];
#endif
	//class methods
	//setup
public:
	
	CAnimManager();


	//process

	//access
	void GetFrame(const EAnim eAnim, const int iTime, const bool bForceUpdate, const bool bIsReversed, EAnimFrame &eAnimFrame, EHeadDirection &eHeadDirection, EDirection &eDirection) const;
	const float GetSpeed(const EAnim eAnim) const;

	const EAnimFrame RandomWalkFrame() const;

	const EHeadDirection RandomHeadDirection(const EDirection eDirection, const bool bIncludeUp = false) const;

	const EDirection RandomDirection() const;
	const bool IsValidHeadDirection(const EHeadDirection eHeadDirection, const EDirection eDirection) const;

	ldwPoint GetScaledLinkToPrevPt(EAnimFrame frame,EAnimPart animPart,EAnimGender gender,int animSet,float scale);
	ldwPoint GetScaledLinkToNextPt(EAnimFrame frame,EAnimPart animPart,EAnimGender gender,int animSet,float scale,ldwImageGrid **ppImage = 0,int *pFrame = 0);

	ldwPoint GetScaledLinkToPrevPt(EHeadDirection headDirection,EAnimGender gender,int animSet,float scale);
	ldwPoint GetScaledLinkToNextPt(EHeadDirection headDirection,EAnimGender gender,int animSet,float scale,ldwImageGrid **ppImage = 0,int *pFrame = 0);

	int        GetHeadFrame(EHeadDirection headDirection,EDirection direction);
	EDirection GetDirection(EAnimFrame animFrame);
	EDirection GetDirection(EHeadDirection headDirection);
	void       GetAnimFrame(EBodyPosition position,EAnimFrame &animFrame,EDirection &direction);

	int GetFrameCount(int animIndex);

	int FindAnim(const char *animName);

	void Save();
	void LoadAssets();

private:

	const EHeadDirection RandomNorthHeadDirection() const;
	const EHeadDirection RandomEastHeadDirection() const;
	const EHeadDirection RandomSouthHeadDirection() const;
	const EHeadDirection RandomWestHeadDirection() const;
};

/******************************************************************************\
** GLOBAL DECLARATIONS
\******************************************************************************/
extern CAnimManager		AnimManager;

#endif

/******************************************************************************\
** END OF FILE
\******************************************************************************/