/******************************************************************************\
**
**     COMPANY:	LDW Software
**
**      MODULE:	Bath 
**
** DESCRIPTION:	Bath Restoration Puzzle
**
**      AUTHOR:	Arthur Humphrey, from template of Adrian Francis
**
\******************************************************************************/

#ifndef BELIEVERIDOLPUZZLE_H
#define BELIEVERIDOLPUZZLE_H

/******************************************************************************\
** LIBRARY INCLUDES
\******************************************************************************/
#include "ldwImage.h"
#include "PuzzleInterface.h"
#include "../Game/theGameState.h"
#include <math.h>

/******************************************************************************\
** CONSTANT DEFINITIONS
\******************************************************************************/

//const ldwPoint Fruitwells_c_sCenter(114, 1802);

enum EBelieverIdolStatus
{
	eBelieverIdol_InitialState = 0,
	eBelieverIdol_WorkStarted,
	//eBelieverIdol_OneQuarterBuilt,
	//eBelieverIdol_OneHalfBuilt,
	//eBelieverIdol_ThreeQuartersBuilt,
	//eBelieverIdol_Built,
	eBelieverIdol_NumberOfStates

};


enum EBelieverIdolStage
{
	m_eBelieverIdolFirstStage = 0,
	m_eBelieverIdolSecondStage,
	m_eBelieverIdolThirdStage
};

/******************************************************************************\
** CLASS DEFINITIONS
\******************************************************************************/
class CBelieverIdolPuzzle : public IPuzzle
{
	friend class CPuzzle;

public:

	CBelieverIdolPuzzle(void);



	
	void Reset();

	const int	LoadState(const unsigned char *pData);
	const int	SaveState(const unsigned char *pData, const int iMaxDataSize) const;
	bool		AdvancePuzzleState();
	bool		IsWorkPermitted();
	EBelieverIdolStatus	 GetStatus();
	int GetPercentComplete();
	EBelieverIdolStage GetStage();

	void Draw() const;
	

	void		Draw(const int iRef);
	const int SaveStateSize() {return sizeof (SSaveState);}
	void		LoadAssets();
	void		ReleaseAssets();

	// process
	void Refresh();
	void Update();
	void Advance(CVillager &Villager, const int iRef);
	void Advance();
	void Complete();

	void InflictEarthQuakeDamage(void);

	// access

private:

	EBelieverIdolStatus m_eBelieverIdolStatus;

	EBelieverIdolStage m_eBelieverIdolStage;
	//class structures
	struct SSaveState
	{
		EBelieverIdolStatus	m_eBelieverIdolStatus;
		EBelieverIdolStage	m_eBelieverIdolStage;
		int					m_iBelieverIdolProgress;
	};
	
	int m_iBelieverIdolProgress;

	ldwUINT32 m_uPlantTimer;
	//class data
	float m_fBeamAlphaSin;
	float m_fBeamAlphaPhased;

	int m_iBeamAlphaCount;
	theGameState  *gs;

	ldwImageGrid *	m_pScaffold;

	ldwImageGrid *	m_pTotem;
	ldwImageGrid *	m_pIdolCoverings;
	ldwImageGrid *  m_pIdolStageStrip;
	ldwImageGrid *  m_pInvertedLigtBeam;
	ldwImageGrid *  m_pInvertedLigtBeamSC;
	ldwImageGrid *  m_pSmallCollectibles;


};

/******************************************************************************\
** GLOBAL DECLARATIONS
\******************************************************************************/
extern CBelieverIdolPuzzle		BelieverIdolPuzzle;

#endif

/******************************************************************************\
** END OF FILE
\******************************************************************************/