/******************************************************************************\
**
**     COMPANY:	LDW Software
**
**      MODULE:	Bath 
**
** DESCRIPTION:	Bath Restoration Puzzle
**
**      AUTHOR:	Arthur Humphrey, from template of Adrian Francis
**
\******************************************************************************/

#ifndef ConvertHeathenChiefPUZZLE_H
#define ConvertHeathenChiefPUZZLE_H

/******************************************************************************\
** LIBRARY INCLUDES
\******************************************************************************/
#include "ldwImage.h"
#include "PuzzleInterface.h"
#include "../Game/theGameState.h"

/******************************************************************************\
** CONSTANT DEFINITIONS
\******************************************************************************/

//const ldwPoint Fruitwells_c_sCenter(114, 1802);

enum EConvertHeathenChiefStatus
{
	eConvertHeathenChief_InitialState = 0,
	eConvertHeathenChief_WorkStarted,
	eConvertHeathenChief_OneQuarterDown,
	eConvertHeathenChief_OneHalfDown,
	eConvertHeathenChief_ThreeQuartersDown,
	eConvertHeathenChief_Down,
	eConvertHeathenChief_NumberOfStates

};


/******************************************************************************\
** CLASS DEFINITIONS
\******************************************************************************/
class CConvertHeathenChiefPuzzle : public IPuzzle
{
	friend class CPuzzle;

public:

	CConvertHeathenChiefPuzzle(void);



	EConvertHeathenChiefStatus m_eConvertHeathenChiefStatus;

	void Reset();

	bool IsNecklacePieceAvailable(EPuzzle puzzle);
	bool IsNecklacePieceInPlace(EPuzzle puzzle);
	void SetNecklacPieceInPlace(EPuzzle puzzle, bool state);
	const int	LoadState(const unsigned char *pData);
	const int	SaveState(const unsigned char *pData, const int iMaxDataSize) const;
	bool		AdvancePuzzleState();
	EConvertHeathenChiefStatus	 GetStatus();
	int GetPercentComplete();
	bool IsNecklaceAvailable();
	int GetNumNecklacePiecesFound(){return numNecklacePiecesCollected;}
		

	void AdjustNumKids(int adjustment);
	void Draw() const;
	

	void		Draw(const int iRef);
	const int SaveStateSize() {return sizeof (SSaveState);}
	void		LoadAssets();
	void		ReleaseAssets();
	bool GetNecklaceComplete();
	void SetNecklaceComplete();


	// process
	void Refresh();
	void Update();
	void Advance(CVillager &Villager, const int iRef);
	void Advance(int howMany);
	void Complete();

	// access

private:
	
	bool MasterFarmerPieceInPlace;
	bool MasterScientistPieceInPlace;
	bool MasterBuilderPieceInPlace;
	bool MasterDoctorPieceInPlace;
	bool allNecklacePiecesInPlace;

	int numNecklacePiecesCollected;
	//class structures
	struct SSaveState
	{
		EConvertHeathenChiefStatus	m_eConvertHeathenChiefStatus;
		int					m_iConvertHeathenChiefProgress;
		bool				MasterFarmerPieceInPlace;
		bool				MasterScientistPieceInPlace;
		bool				MasterBuilderPieceInPlace;
		bool				MasterDoctorPieceInPlace;
		bool				allNecklacePiecesInPlace;
	};
	
	int m_iConvertHeathenChiefProgress;
	//int m_iHowManyChildrenInTotem;
	
	//class data
	theGameState  *gs;

	ldwImageGrid *	m_pScaffold;
	
	ldwUINT32 	m_u5thChildTimer;
	
	ldwImageGrid *	m_pTotem;

	ldwImageGrid *	m_pBlinkyEyes;

	ldwImageGrid *	m_pFarmerPiece;

	ldwImageGrid *	m_pScientistPiece;

	ldwImageGrid *	m_pBuilderPiece;

	ldwImageGrid *	m_pDoctorPiece;

	ldwImageGrid *	m_pCompletedNecklace;

	ldwImageGrid *	m_pChiefMask;

	ldwImageGrid *	m_pMastersMask;

};

/******************************************************************************\
** GLOBAL DECLARATIONS
\******************************************************************************/
extern CConvertHeathenChiefPuzzle		ConvertHeathenChiefPuzzle;

#endif

/******************************************************************************\
** END OF FILE
\******************************************************************************/