/******************************************************************************\
**
**     COMPANY:	LDW Software
**
**      MODULE:	Environment
**
** DESCRIPTION:	Manages environmental effects
**
**      AUTHOR:	Adrian Francis
**
\******************************************************************************/

#ifndef ENVIRONMENT_H
#define ENVIRONMENT_H

/******************************************************************************\
** LIBRARY INCLUDES
\******************************************************************************/
#include "ldwImage.h"
#include "../Game/TechObserverInterface.h"

/******************************************************************************\
** CONSTANT DEFINITIONS
\******************************************************************************/

const ldwPoint Environment_c_sAlchemyLabPos(225, 1220);
const ldwPoint Environment_c_sScienceLabPos(28, 416);

enum EPropEnum
{
	eProp_Start   = 0,
	eProp_CrabToMossyRocks = eProp_Start,
	eProp_SoapyPond,
	eProp_Flower1,
	eProp_Flower2,
	eProp_Flower3,
	eProp_Flower4,
	eProp_Flower5,
	eProp_Flower6,
	eProp_Flower7,
	eProp_Flower8,
	eProp_Flower9,
	eProp_Flower10,
	eProp_Wasps,
	eProp_FrogJoke,
	eProp_IdolPolishScaffold,
	eProp_IdolResourceArea,
	eProp_ProofGoodPartial,
	eProp_ProofCrappyPartial,
	eProp_ProofGoodComplete,
	eProp_ProofCrappyComplete,
	eProp_ProofDisapprove,
	eProp_BelieverBuildingChallengeFinal,
	eProp_BelieverBuildingChallengeStage3,
	eProp_BelieverBuildingChallengeStage2,
	eProp_BelieverBuildingChallengeStage1,
	eProp_HeathenBuildingChallengeFinal,
	eProp_HeathenBuildingChallengeStage3,
	eProp_HeathenBuildingChallengeStage2,
	eProp_HeathenBuildingChallengeStage1,
	eProp_PanPickedUpInvisible,
	eProp_BallPickedUpInvisible,
	eProp_PotPickedUpInvisible,
	eProp_PanRetrievedInvisible,
	eProp_InhibitFarmerNPiece,
	eProp_InhibitScientistNpiece,
	eProp_InhibitBuilderNPiece,
	eProp_InhibitDoctorNPiece,
	eProp_InhibitCompletedNecklace,
	e_MaxNumProps
};

enum ECarryPropEnum
{
	e_CarryPropdebris,
	e_MaxNumCarryProps
};

const int cMaxPropObservers = 2;

/******************************************************************************\
** CLASS DEFINITIONS
\******************************************************************************/

class IPropObserver
{
public:

	virtual void PropExpired(EPropEnum prop) = 0;
};		

class CEnvironment : ITechObserver
{
public:
	
	struct PropTypeStruct
	{
		bool      	   activated;
		ldwUINT32 	   timer;
		IPropObserver *pObserver[cMaxPropObservers];
		int            observerCount;

	};

	struct CarryPropElementStruct
	{
		ldwUINT32 timer;
		ldwPoint sourceLocation;
		ldwPoint destinationLocation;
		ldwImageGrid *item;
	};

	struct CarryPropSequenceStruct
	{
		CarryPropElementStruct itemList[8];
		bool      activated;
		ldwUINT32 timer;
		ldwPoint location;
		int RequiredVisitsPerItem; 
		int numVisitsThisItem;
		int currentItem;
	};

	struct CarryPropTypeMasterStruct
	{
		CarryPropSequenceStruct carryPropLibrary[e_MaxNumCarryProps];
		ldwUINT32 timer;
		ldwPoint location;
		int requiredVisits;

	};

	static PropTypeStruct PropArray[e_MaxNumProps];
	//CarryPropTypeMasterStruct CarryPropArray[e_MaxNumCarryProps][8];

	CEnvironment(void);


	void InitProps(); 

	void InitCarryProps();
	void SetProp(EPropEnum whichProp);
	void ClearProp(EPropEnum whichProp);
	void UpdateProps();
	bool GetPropState(EPropEnum);
	const bool PropIsActive(EPropEnum whichProp) const;
	void RefreshProps();

	static void WatchProp(EPropEnum prop,IPropObserver *observer);

	void Reset(bool alsoResetProps = true);
	void LoadAssets();
	void ReleaseAssets();

	// process
	void Refresh();
	void Update();
	void Draw();
	void Draw(const int iRef) const;

	void TechAdvanced(const ETech eTech, const int iLevel);

private:

	//class enumerations
	enum ERef
	{
		eRef_AlchemyLab,
		eRef_MagicHerb,
		eRef_NatureHerb,
		eRef_ScienceLab
	};

	//class data

};

/******************************************************************************\
** GLOBAL DECLARATIONS
\******************************************************************************/
extern CEnvironment		Environment;

#endif

/******************************************************************************\
** END OF FILE
\******************************************************************************/