/******************************************************************************\
**
**     COMPANY:	LDW Software
**
**      MODULE:	Bath 
**
** DESCRIPTION:	Bath Restoration Puzzle
**
**      AUTHOR:	Arthur Humphrey, from template of Adrian Francis
**
\******************************************************************************/

#ifndef FENCEPUZZLE_H
#define FENCEPUZZLE_H

/******************************************************************************\
** LIBRARY INCLUDES
\******************************************************************************/
#include "ldwImage.h"
#include "PuzzleInterface.h"
#include "../Game/theGameState.h"

/******************************************************************************\
** CONSTANT DEFINITIONS
\******************************************************************************/

//const ldwPoint Fruitwells_c_sCenter(114, 1802);

enum EFenceRecycleStatus
{
	eFence_FenceInitialState = 0,
	eFence_FenceWorkStarted,
	eFence_FenceOneQuarterDown,
	eFence_FenceOneHalfDown,
	eFence_FenceThreeQuartersDown,
	eFence_FenceDown,
	eFence_GraneryFinished,
	eFence_NumberOfStates

};

/******************************************************************************\
** CLASS DEFINITIONS
\******************************************************************************/
class CFencePuzzle : public IPuzzle
{
	friend class CPuzzle;

public:

	CFencePuzzle(void);



	EFenceRecycleStatus m_eFenceRecycleStatus;

	void Reset();

	const int	LoadState(const unsigned char *pData);
	const int	SaveState(const unsigned char *pData, const int iMaxDataSize) const;
	bool		AdvancePuzzleState();
	EFenceRecycleStatus	 GetStatus();
	int GetPercentComplete();//{return m_iFenceProgress;}
	//int GetDebrisRemaining() {return m_iDebrisRemaining;}

	void Draw() const;
	

	void		Draw(const int iRef);
	const int SaveStateSize() {return sizeof (SSaveState);}
	void		LoadAssets();
	void		ReleaseAssets();

	// process
	void Refresh();
	void Update();
	void Advance(CVillager &Villager, const int iRef);
	void Advance(EFenceRecycleStatus currentStatus);
	void Complete();

	// access
	const bool IsRepaired() const;

private:
	

	//class structures
	struct SSaveState
	{
		EFenceRecycleStatus	m_eFenceRecycleStatus;
		//ldwUINT32			m_uPlantTimer;
		//bool				m_bBfliesDeployed;
		int					m_iFenceProgress;
		int					m_iGranaryProgress;
	};
	
	int m_iFenceProgress;
	int m_iGranaryProgress;
	ldwUINT32 m_uPlantTimer;
	//class data
	

	theGameState  *gs;

	ldwImageGrid *	m_pScaffold;
	ldwImageGrid *	m_pFish;

	ldwImageGrid *	m_pFoodBin1;
	ldwImageGrid *	m_pFoodBin2;
	ldwImageGrid *	m_pFoodBin3;
	ldwImageGrid *	m_pFoodBin4;
	ldwImageGrid *  m_pFoodBinStages;
	ldwImageGrid *  m_pFoodBin;
	ldwImageGrid *	m_pFenceStrip;


};

/******************************************************************************\
** GLOBAL DECLARATIONS
\******************************************************************************/
extern CFencePuzzle		RecycleFencePuzzle;

#endif

/******************************************************************************\
** END OF FILE
\******************************************************************************/