//
//		Definition of the main game class
//
#ifndef THE_GOD_POWER_MANAGER_H
#define THE_GOD_POWER_MANAGER_H

#include "ldwTypes.h"
#include "../Sound/Sound.h"
#include "../Utils/theStringManager.h"

class ldwHintControl;


enum EGodPower
{
	eGodPower_None = -1,
	eGodPower_SpawnButterflies,
	eGodPower_SwarmOfBees,
	eGodPower_Sunshine,
	eGodPower_LightningStrike,
	eGodPower_HandOfBloom,
	eGodPower_RainStorm,
	eGodPower_FogOfDoom,
	eGodPower_TimeWarp,
	eGodPower_Reanimate,
	eGodPower_GrantYouth,
	eGodPower_EarthQuake,

	eGodPower_NumberOf
};

//TODO: Combine the following arrays with the traits array, and remove unnecessary data (icon pos, power) from traits
const int GodPowerManager_c_iCost[eGodPower_NumberOf] = {
	10,		//eGodPower_SpawnButteflies
	25,		//eGodPower_SwarmOfBees
	50,		//eGodPower_Sunshine
	100,	//eGodPower_LightningStrike
	150,	//eGodPower_HandOfBloom
	300,	//eGodPower_RainStorm
	400,	//eGodPower_FogOfDoom
	500,	//eGodPower_TimeWarp
	600,	//eGodPower_Reanimate
	700,	//eGodPower_GrantYouth
	800,	//eGodPower_EarthQuake
};

const StringId GodPowerManager_c_eHint[eGodPower_NumberOf] = {

	eString_SpawnButterfliesHint,
	eString_SwarmOfBeesHint,
	eString_SunshineHint,
	eString_LightningStrikeHint,
	eString_HandOfBloomHint,
	eString_RainStormHint,
	eString_FogOfDoomHint,
	eString_TimeWarpHint,
	eString_ReanimateHint,
	eString_GrantYouthHint,
	eString_EarthquakeHint,
};

const int cSwarmOfBeesRadius = 150;
const int cHandOfBloomRadius = 104;
const int cLightningRadius = 304;

enum
{
//	ePowerStartX   = -40,
	ePowerStartX   = 25,
	ePowerStartY   = 50,
	ePowerRectSize = 64,
	ePowerXSpacing = ePowerRectSize + 26,

	/* this segement for right-to-left spell placement
	ePowerStartX   = 770,
	ePowerStartY   = 50,
	ePowerRectSize = 60,
	ePowerXSpacing = ePowerRectSize + 30,
	*/
};

static struct sGodPowerTraits
{
	EGodPower power;
	ESound	  soundId;
	int		  radiusOfInfluence;
	bool	  showTargetArea;
	bool	  m_bRequiresTargetVillager;
	int		  iconX;
	int 	  iconY;
}
GodPowerTraits[eGodPower_NumberOf] =
{
	{eGodPower_SpawnButterflies,eSound_Butterlies,50,false,false,ePowerStartX + ePowerXSpacing * eGodPower_SpawnButterflies,ePowerStartY},
	{eGodPower_SwarmOfBees,eSound_Bees,cSwarmOfBeesRadius,true,false,ePowerStartX + ePowerXSpacing * eGodPower_SwarmOfBees,ePowerStartY},
	{eGodPower_Sunshine,eSound_ChimesVV5,0,true,false,ePowerStartX + ePowerXSpacing * eGodPower_Sunshine,ePowerStartY},
	{eGodPower_LightningStrike,eSound_LightningStrike,300,false,false,ePowerStartX + ePowerXSpacing * eGodPower_LightningStrike,ePowerStartY},
	{eGodPower_HandOfBloom,eSound_Rustling,cHandOfBloomRadius,true,false,ePowerStartX + ePowerXSpacing * eGodPower_HandOfBloom,ePowerStartY},
	{eGodPower_RainStorm,eSound_Wind,0,true,false,ePowerStartX + ePowerXSpacing * eGodPower_RainStorm,ePowerStartY},
	{eGodPower_FogOfDoom,eSound_Wind,0,true,false,ePowerStartX + ePowerXSpacing * eGodPower_FogOfDoom,ePowerStartY},
	{eGodPower_TimeWarp,eSound_Ticktock,0,false,true,ePowerStartX + ePowerXSpacing * eGodPower_TimeWarp,ePowerStartY},
	{eGodPower_Reanimate,eSound_AngelicChorus,0,false,true,ePowerStartX + ePowerXSpacing * eGodPower_Reanimate,ePowerStartY},
	{eGodPower_GrantYouth,eSound_Ticktock,0,false,true,ePowerStartX + ePowerXSpacing * eGodPower_GrantYouth,ePowerStartY},
	{eGodPower_EarthQuake,eSound_Earthquake,0,true,false,ePowerStartX + ePowerXSpacing * eGodPower_EarthQuake,ePowerStartY},
};

class CGodPowerSpell;

class CGodPowerManager 
{
public:

	CGodPowerManager();
	~CGodPowerManager();

	bool ActivatePower(EGodPower power,ldwUINT32 expiration = 0);
	void DeactivatePower(EGodPower power);
	void UpdateAvailablePowers();

	void Reset();
	void ReleaseAssets();

	void Update();
	void Draw();
	void DrawInWorld();
	void Draw(int iRef);
	void DrawTargetArea();

	void FrameUpdate();

	const bool HandleMouseDown(const ldwPoint sMousePos);
	const bool HandleMouseUp(const ldwPoint sMousePos);
	const bool HandleMouseMove(const ldwPoint sMousePos);

	int PowerInHand() {return mPowerInUseIndex != -1;}
	void DropPower(void) { mPowerInUseIndex = -1; }
	const int GetHighlightedMana(void);

private:

	enum
	{
		eMaxActivePowers = eGodPower_NumberOf,		// No need for this? Caused memory corruption when the two values differed - AF
		eMaxActiveSpells = 10,
	};

	void ClearSpells();

	void DrawPower(EGodPower power,int xPos,int yPos);
	bool DeployPower(EGodPower power,ldwPoint where);
	int  FindEmptySpellSlot(EGodPower replacePower = eGodPower_None);

	ldwRect GetUIRect(EGodPower power);
	void    SetHintText(EGodPower power,bool clearHint = false);

	struct sActivePower
	{
		EGodPower  power;
		ldwUINT32  expiration;
		bool	   hasCursor;
	}
	mActivePower[eMaxActivePowers];

	CGodPowerSpell *mActiveSpell[eMaxActiveSpells];

	ldwHintControl *mHint[eGodPower_NumberOf];

	int mActivePowerCount;

	int 	  mPowerInUseIndex;
	ldwUINT32 mPowerInUseFxTime;

	int mCursorOnIndex;
	int mMouseDownIndex;

	float mEffectOcillationAngle;
	float mEffectOcillationIncrement;

	int m_iHighlightedMana;
};

extern CGodPowerManager		GodPower;

#endif