/******************************************************************************\
**
**     COMPANY:	LDW Software
**
**      MODULE:	HotSpot
**
** DESCRIPTION:	Dispatches behaviours when a sprite is dropped in a hotspot
**
**
\******************************************************************************/

#ifndef HOTSPOT_H
#define HOTSPOT_H

/******************************************************************************\
** LIBRARY INCLUDES
\******************************************************************************/
#include "ContentMap.h"
#include "../Game/Isola.h"	//EDirection
#include "../Villagers/Villager.h"

/******************************************************************************\
** CONSTANT DEFINITIONS
\******************************************************************************/
const int HotSpot_c_iMaxHotSpots = CContentMap::eHotSpot_NumberOf;


/******************************************************************************\
** TYPE DEFINITIONS
\******************************************************************************/

enum EHotSpotEvent
{
	eHotSpotEvent_VillagerDrop,
    eHotSpotEvent_VillagerHover,
    eHotSpotEvent_MapClick
};

enum EHotSpotPermission
{
	eHotSpotPermission_Workers, 
	eHotSpotPermission_Anyone,
	eHotSpotPermission_NonMother,
};

typedef const bool (* LPHOTSPOTHANDLER)(EHotSpotEvent event,CVillager *pVillager);

/******************************************************************************\
** CLASS DEFINITIONS
\******************************************************************************/
class CHotSpot
{
public:

	static void SetHandler(const CContentMap::EHotSpot eHotSpot, 
						   LPHOTSPOTHANDLER pHandler,
						   EDirection eBias = eDirection_None,
						   EHotSpotPermission permission = eHotSpotPermission_Workers);

	CHotSpot(void);

	// process

	bool HandleVillagerDrop(CVillager *pVillager, bool priorityHotspots = false);
	bool HandleMapClick(ldwPoint worldPoint);
	bool HandleVillagerHover(CVillager *pVillager);

	const EDirection Bias(const CContentMap::EHotSpot eHotSpot);

private:

	static const bool Kindling(EHotSpotEvent event,CVillager *pVillager);
	static const bool Firewood(EHotSpotEvent event,CVillager *pVillager);
	static const bool FirePlace(EHotSpotEvent event,CVillager *pVillager);
	static const bool Foodbin(EHotSpotEvent event,CVillager *pVillager);
	static const bool House1(EHotSpotEvent event,CVillager *pVillager);
	static const bool House2(EHotSpotEvent event,CVillager *pVillager);
	static const bool House3(EHotSpotEvent event,CVillager *pVillager);
	static const bool LoveShackHandler(EHotSpotEvent event,CVillager *pVillager);
	static const bool ClothingHutHandler(EHotSpotEvent event,CVillager *pVillager);
	static const bool RosterOfTheDeadHandler(EHotSpotEvent event,CVillager *pVillager);
	static const bool ResearchHotspot(EHotSpotEvent event,CVillager *pVillager);
	static const bool HospitalLab(EHotSpotEvent event,CVillager *pVillager);
	static const bool BlackberryBushHandler(EHotSpotEvent event,CVillager *pVillager);
	static const bool FruitTreeHandler(EHotSpotEvent event,CVillager *pVillager);
	static const bool PuddleHandler(EHotSpotEvent event,CVillager *pVillager);
	static const bool WestDivingPlatformHandler(EHotSpotEvent event,CVillager *pVillager);
	static const bool EastDivingPlatformHandler(EHotSpotEvent event,CVillager *pVillager);
	static const bool PoolsHandler(EHotSpotEvent event,CVillager *pVillager);
	static const bool OceanHandler(EHotSpotEvent event,CVillager *pVillager);
	static const bool BelieverIdolHandler(EHotSpotEvent event,CVillager *pVillager);
	static const bool HydroFarmHandler(EHotSpotEvent event,CVillager *pVillager);
	static const bool AqueductHandler(EHotSpotEvent event,CVillager *pVillager);
	static const bool LakeHandler(EHotSpotEvent event,CVillager *pVillager);
	static const bool HospitalLabHandler(EHotSpotEvent event,CVillager *pVillager);

	//class structures
	struct SHotSpot
	{
		LPHOTSPOTHANDLER	m_pHandler;
		EDirection			m_eBias;
		EHotSpotPermission	m_permission;
	};

	//class data
	static SHotSpot	m_sHotSpot[HotSpot_c_iMaxHotSpots];
};

/******************************************************************************\
** GLOBAL DECLARATIONS
\******************************************************************************/
extern CHotSpot		HotSpot;

#endif

/******************************************************************************\
** END OF FILE
\******************************************************************************/