/******************************************************************************\
**
**     COMPANY:	LDW Software
**
**      MODULE:	LightningEffect
**
** DESCRIPTION:	Manages a number of LightningEffect emitters
**
**      AUTHOR:	Adrian Francis
**
\******************************************************************************/

#ifndef LIGHTNINGEFFECT_H
#define LIGHTNINGEFFECT_H

/******************************************************************************\
** LIBRARY INCLUDES
\******************************************************************************/
#include "ldwTypes.h"
#include "FloatingAnim.h"
#include "../Utils/SceneManager.h"
#include "../Villagers/VillagerAttachmentInterface.h"

/******************************************************************************\
** CONSTANT DEFINITIONS
\******************************************************************************/
const int LightningEffect_c_iMaxEmitters = 16;
const int LightningEffect_c_iMaxLightningBoltsPerEmitter = 32;

/******************************************************************************\
** CLASS DEFINITIONS
\******************************************************************************/
class CLightningEffect : public IVillagerAttachment
{
public:

	//class enumerations
	enum EType
	{
		eType_Globe,
		eType_Fan
	};

private:

	//class structures
	struct SEmitter
	{
		bool			m_bIsActive;
		EType			m_eType;
		ldwPoint		m_sPos;
		int				m_iSize;
		ldwColor		m_sColor;
		int				m_iThickness;
		EScenePriority	m_eScenePriority;
		int				m_iSceneOffset;
		ldwPoint		m_sBaseOffset;

		int				m_iLightningBolt[LightningEffect_c_iMaxLightningBoltsPerEmitter];
		int				m_iNumLightningBolts;

		public:		const int AddBolt(const ldwPoint sPos, const int iLength, const float fDirection, const ldwColor sColor, const int iThickness, const EScenePriority eScenePriority, const int iSceneOffset = 0);
	};

	//class data
	SEmitter			m_sEmitter[LightningEffect_c_iMaxEmitters];

	//class methods
	//setup
	public:		CLightningEffect(void);
	public:		void Reset();

	// process
	public:		const int Add(const EType eType, const ldwPoint sPos, const int iSize, const ldwColor sColor, const int iThickness, const EScenePriority eScenePriority, const int iSceneOffset = 0, const ldwPoint sBaseOffset = ldwPoint(0, 0));
	public:		void Remove(const int iEmitter);

	public:		void Update(void);

	// access
	public:		void SetPos(const int iEmitter, const ldwPoint sPos);
};

/******************************************************************************\
** GLOBAL DECLARATIONS
\******************************************************************************/
extern CLightningEffect		LightningEffect;

#endif

/******************************************************************************\
** END OF FILE
\******************************************************************************/