/******************************************************************************\
**
**     COMPANY:	LDW Software
**
**      MODULE:	Puzzle
**
** DESCRIPTION:	Maintains completion status of puzzles
**
**
\******************************************************************************/

#ifndef PUZZLE_H
#define PUZZLE_H

/******************************************************************************\
** LIBRARY INCLUDES
\******************************************************************************/
#include "PuzzleInterface.h"

/******************************************************************************\
** CONSTANT DEFINITIONS
\******************************************************************************/
//#define PUZZLE_ENABLE_HOSPITAL

/******************************************************************************\
** CLASS ENUMERATIONS
\******************************************************************************/
enum EPuzzle
{
	// Essential puzzles
	ePuzzle_NurserySchool,
	ePuzzle_ConvertMasterDoctor,
	ePuzzle_RecycleFence,
	ePuzzle_FoodTotem,
	ePuzzle_ResearchTotem,
	ePuzzle_HydroFarm,
	ePuzzle_RestoreMausoleum,
	ePuzzle_BuildingTotem,
	ePuzzle_ChildrensTotem,
	ePuzzle_ConvertMasterBuilder,
	ePuzzle_RestoreLake,
	ePuzzle_MedicineTotem,
	ePuzzle_RainbowTotem,
	ePuzzle_ConvertMasterScientist,
	ePuzzle_BuildBelieverIdol,
	ePuzzle_ClosedEyeTotem,
	ePuzzle_ConvertHeathenChief,
	ePuzzle_HeathenMommy,
	ePuzzle_NumberOfEssential,	

	// Additional puzzles
	ePuzzle_FirePit = ePuzzle_NumberOfEssential,
	ePuzzle_House1Construction,
	ePuzzle_House2Construction,
	ePuzzle_House3Construction,
	ePuzzle_ClothingHutConstruction,
	ePuzzle_LoveShackConstruction,
	ePuzzle_NurseryConstruction,
	ePuzzle_ConvertLastHeathen,

#ifdef PUZZLE_ENABLE_HOSPITAL
	ePuzzle_HospitalConstruction,
#endif

	ePuzzle_NumberOf
};

/******************************************************************************\
** CLASS DEFINITIONS
\******************************************************************************/
class CPuzzle
{
public:

	//class methods
	//setup

	static void RegisterPuzzle(IPuzzle *puzzle,EPuzzle puzzleIndex,int pointsRequired = 1);

	CPuzzle(void);

	void Reset();
	void Reset(const EPuzzle ePuzzle);
	void LoadAssets();
	void ReleaseAssets();

	const int  GetPuzzleStateSize() const;
	const bool LoadState(const unsigned char *pData);
	const bool SaveState(const unsigned char *pData, const int iMaxDataSize) const;

	// process
	void Refresh() const;
	void Refresh(const EPuzzle ePuzzle);
	void Draw() const;
	void Draw(const int iRef);
	void Update();
	void Advance(const EPuzzle ePuzzle, CVillager &Villager, const int iRef);
	void CheckAdvance(EPuzzle puzzle,CVillager &Villager,int stage);

	// access
	void IncrementProgress(const EPuzzle ePuzzle);
	void DecrementProgress(const EPuzzle ePuzzle, const int iPoints = 1);
	const bool IsComplete(const EPuzzle ePuzzle) const;
	const int PercentComplete(const EPuzzle ePuzzle) const;
	const int PuzzlesComplete() const;
	const int EssentialPuzzlesComplete() const;
	const int CompletionTime(const EPuzzle ePuzzle) const;

	const int PointsRemaining(const EPuzzle ePuzzle) const;
	void SetPointsRemaining(const EPuzzle ePuzzle, const int iPointsRemaining);
	const int Points(const EPuzzle ePuzzle) const;

	const int PointsRequired(const EPuzzle puzzle) {return scm_iPointsRequired[puzzle];}

private:

	//class structures
	struct SPuzzle
	{
		int				m_iPoints;
		unsigned int	m_uiCompletionTime;
	};

	//class data
	static int			scm_iPointsRequired[ePuzzle_NumberOf];
	static IPuzzle *	sm_pPuzzle[ePuzzle_NumberOf];

	SPuzzle				m_sPuzzle[ePuzzle_NumberOf];
};

/******************************************************************************\
** GLOBAL DECLARATIONS
\******************************************************************************/
extern CPuzzle		Puzzle;

#endif

/******************************************************************************\
** END OF FILE
\******************************************************************************/