/******************************************************************************\
**
**     COMPANY:	LDW Software
**
**      MODULE:	RestoreLake
**
** DESCRIPTION:	Restore lake puzzle
**
\******************************************************************************/

#ifndef RESTORELAKE_H
#define RESTORELAKE_H

/******************************************************************************\
** LIBRARY INCLUDES
\******************************************************************************/
#include "ldwTypes.h"
#include "PuzzleInterface.h"
#include "Puzzle.h"

/******************************************************************************\
** CONSTANT DEFINITIONS
\******************************************************************************/
const ldwPoint RestoreLake_c_sLakePos(1320, 1768);
const int RestoreLake_c_iSecondsUntilFloodAfterCastingRainStorm = 120;

/******************************************************************************\
** CLASS DEFINITIONS
\******************************************************************************/
class CRestoreLake : public IPuzzle
{
	friend class CPuzzle;

public:

	CRestoreLake(void);

	void Reset();

	void LoadAssets();
	void ReleaseAssets();

	const int LoadState(const unsigned char *pData);
	const int SaveState(const unsigned char *pData, const int iMaxDataSize) const;

	const int SaveStateSize() {return sizeof (SSaveState);}

	// process

	void Refresh();
	void Draw() const;
	void Draw(const int iRef);
	void Update();
	void Complete();

	void SpawnFloppingFish(void);
	void SpawnJumpingFish(void);
	void StartFloodCountdown(void);

	// access
	const bool IsLakePermanentlyFlooded(void) const;

private:

	//class structures
	struct SSaveState
	{
		unsigned int	m_uiTimeToFlood;
	};

	//class data
	unsigned int	m_uiTimeToFlood;
	int				m_iPrevWetness;
};

/******************************************************************************\
** GLOBAL DECLARATIONS
\******************************************************************************/
extern CRestoreLake		RestoreLake;

#endif

/******************************************************************************\
** END OF FILE
\******************************************************************************/