/******************************************************************************\
**
**     COMPANY:	LDW Software
**
**      MODULE:	VillagerSkills
**
** DESCRIPTION:	Villager skills
**
**
\******************************************************************************/

#ifndef VILLAGER_SKILLS_H
#define VILLAGER_SKILLS_H

/******************************************************************************\
** LIBRARY INCLUDES
\******************************************************************************/
#include "SkillTypes.h"
#include "ldwGameState.h"

class CVillager;

/******************************************************************************\
** CONSTANT DEFINITIONS
\******************************************************************************/
const int VillagerSkills_c_iTraineeLevel = 20;
const int VillagerSkills_c_iAdeptLevel = 50;
const int VillagerSkills_c_iMasterLevel = 88;
const int VillagerSkills_c_iPredisposedLevel = 1;

/******************************************************************************\
** TYPE DEFINITIONS
\******************************************************************************/
struct SSkillData
{
};


/******************************************************************************\
** CLASS DEFINITIONS
\******************************************************************************/
class CVillagerSkills
{
public:

	CVillagerSkills();
	void Copy(CVillagerSkills &Skills);
	void Reset();

	void Update(CVillager &Villager);

	void      SetSkill(const ESkill eSkill, const int iLevel) { m_iSkillLevel[eSkill] = (float) iLevel; if (m_iSkillLevel[eSkill] > 100) m_iSkillLevel[eSkill] = 100; else if (m_iSkillLevel[eSkill] < 0) m_iSkillLevel[eSkill] = 0; }
	const int Skill(const ESkill eSkill) const { return (int) m_iSkillLevel[eSkill]; }

	void AdjustSkill(const ESkill eSkill, const int iAdj) { AdjustSkill(eSkill,(float) iAdj); }
	void AdjustSkill(const ESkill eSkill, const float fAdj);

	void SetPreferredSkill(const ESkill ePreferredSkill) { m_ePreferredSkill = ePreferredSkill; }
	const ESkill PreferredSkill() const { return m_ePreferredSkill; }

	const ESkill RandomSkill() const { return (ESkill)randN(eSkill_NumberOf); }
	const ESkill BestSkill(int *piBestLevel = 0) const;
	const int TotalSkill() const;
	const int SkillsMastered() const;

	const bool IsEsteemedElder() const;

	const bool IsTraineeOf(ESkill eSkill) const { return m_iSkillLevel[eSkill] >= VillagerSkills_c_iTraineeLevel; }
	const bool IsAdeptAt(ESkill eSkill) const { return m_iSkillLevel[eSkill] >= VillagerSkills_c_iAdeptLevel; }
	const bool IsMasterOf(ESkill eSkill) const { return m_iSkillLevel[eSkill] >= VillagerSkills_c_iMasterLevel; }
	const bool IsMasterOfAny() const;
	const bool IsTraineeLevel(const int iLevel) const { return iLevel >= VillagerSkills_c_iTraineeLevel; }
	const bool IsAdeptLevel(const int iLevel) const { return iLevel >= VillagerSkills_c_iAdeptLevel; }

	const bool IsMasterLevel(const int iLevel) const { return iLevel >= VillagerSkills_c_iMasterLevel; }

	const bool IsInexperiencedAt(const ESkill eSkill) const { return m_iSkillLevel[eSkill] == 0; }
	const bool IsInclinedTo(const ESkill eSkill) const { return m_iSkillLevel[eSkill] >= VillagerSkills_c_iPredisposedLevel; }

private:

	//class data
	float		m_iSkillLevel[eSkill_NumberOf];
	ESkill		m_ePreferredSkill;
};

#endif

/******************************************************************************\
** END OF FILE
\******************************************************************************/