//
//		Definition of the main game class
//
#ifndef THE_ALIGN_VILLAGER_SCENE_H
#define THE_ALIGN_VILLAGER_SCENE_H

#include "../Villagers/AnimControl.h"
#include "../Villagers/AnimManager.h"

#include "ldwScene.h"

class ldwButton;
class ldwSoundTrack;
class ldwSoundFx;
class theGameState;
class ldwTextControl;
class theVariableMeter;
class theStringManager;

class CAlignAnimScene;
class CEditAnimScene;
class ldwFont;
class ldwImageGrid;

class theAlignVillagerScene : public ldwScene
{
public:

	theAlignVillagerScene();
	~theAlignVillagerScene();

	bool HandleKey(int messageId,int keyCode,int character);
	bool HandleMouse(int messageId,ldwPoint location);
	bool HandleMessage(int messageId,long data);
//	bool ForwardWaitingMessage(StringId whichMessage);

	void Activate(bool active);

protected:

	void DrawScene();
	void DrawSceneFinal();
	void UpdateScene();

private:

	void DrawSceneAlignment();
	void DrawSceneEditAnimation();

	void SetupAnimation();				  
	void UpdateCurrentSetLabel();
	void UpdateAnimationParamDisplay();

	bool PixelIsVisible(ldwImageGrid *image,int row,int column,ldwPoint checkPt);

	void DrawBaseLine(ldwRect &drawRect,ldwColor color);
	void DrawCross(ldwPoint centerPt,ldwColor color,int size);

	void UpdateAnimParams();
	void UpdateAnimation(EHeadDirection headFrame,EAnimFrame animFrame,int targetFrame,bool insertBefore = false);
	void DeleteAnimFrame(int targetFrame);


	void CopyLinks(CAnimManager::SAnimImageDef *pDestAnimImage,CAnimManager::SAnimImageDef *pSourceAnimImage, int sourceSet);

	void NewAnimation();
	void DeleteAnimation();

	enum 
	{
		eDoneButtonId = 1,
		eSaveButtonId,
		eAnimDownButtonId,
		eAnimUpButtonId,
		eSetDownButtonId,
		eHeadSetUpButtonId,
		eHeadSetDownButtonId,
		eSetUpButtonId,
		eMaleRadioButtonId,
		eFemaleRadioButtonId,
		eAnimFrameLeftButtonId,
		eAnimFrameRightButtonId,
		eWalkingBodiesDownButtonId,
		eWalkingBodiesUpButtonId,
		eActionBodiesDownButtonId,
		eActionBodiesUpButtonId,
		eHeadFrameUpButtonId,
		eHeadFrameDownButtonId,
		eAnimEffectDownButtonId,
		eAnimEffectUpButtonId,
		eAnimFpsUpButtonId,
		eAnimFpsDownButtonId,
		eEditAnimNewButtonId,
		eEditAnimDeleteButtonId,

		eAlignmentTab = 0,
		eEditAnimationTab,
		
		eMaxFrames = 16,eBlinkTime = 5,eBlinkWindow = 10,eAllFrames = -1,

		eKnobSize = 11,eLeftMargin = eKnobSize,

		eBodiesPerRow = 8, eBodyRows = 1,
		eHeadsPerRow = 4, eHeadRows = 4,

		eEnd
	};

	ldwFont *smallFont;

	int currentTab;

	bool hideHeads;

	CAlignAnimScene *alignAnimScene;
	CEditAnimScene  *editAnimScene;

	ldwRect alignTabRect;
	ldwRect editTabRect;

	ldwPoint anim1FrameOffset[eMaxFrames];
	ldwPoint anim2FrameOffset[eMaxFrames];
	ldwPoint anim3FrameOffset[eMaxFrames];

	bool headLinkDrag;
	bool headLinkHighlight;

	bool anim1Drag;
	bool anim1Highlight;

	bool anim2Drag;
	bool anim2Highlight;

	bool anim3Drag;
	bool anim3Highlight;

	bool gridLineXDrag;
	bool gridLineYDrag;

	int  activeFrame;

	ldwRect gridLineXRect;
	ldwRect gridLineYRect;

	ldwRect headRect;
	ldwRect allFramesRect;
	ldwRect frameRect[eMaxFrames];
	ldwRect animateRect;

	int headFrame;
	int imageFrame[eMaxFrames];

	int 		selectedAnimation;
	int 		selectedSet;
	int 		selectedHeadSet;
	EAnimGender selectedGender;

	int maxVisibleFrames;
	int maxFramesShowing;

	int firstFrameShowing;

	int frameCount;
	int editFrameCount;

	ldwRect animRect;

	ldwUINT32 animTime;
	ldwUINT32 animStartTime;

	ldwPoint startLine;
	ldwPoint endLine;

	CAnimManager::SFrameDef *pFrameDef[eMaxFrames];

	EAnimFrame animFrame[eMaxFrames];

	float mAnimAdjustScale;

	int anim1Frame[eMaxFrames];
	int anim1Set;
	int anim2Frame[eMaxFrames];
	int anim2Set;
	int anim3Frame[eMaxFrames];
	int anim3Set;

	int maxFrameWidth;
	int bodyWidth;
	int bodyHeight;
	int walkingBodyCount;
	int walkingBodiesStartFrame;
	int actionBodyCount;
	int sitLieBodyCount;
	int actionBodiesStartFrame;
	int headWidth;
	int headHeight;
	int headCount;
	int headRows;

	ldwRect	walkingBodiesRect;
	ldwRect actionBodiesRect;
	ldwRect headsRect;

	int  dragFrame;

	bool dragWalkingBody;
	bool dragActionBody;
	bool dragHead;

	bool onEditDropTarget;
	int  editTargetFrame;
	bool editDropInsertFrame;

	ldwImageGrid *anim1Image[eMaxFrames];
	ldwImageGrid *anim2Image[eMaxFrames];
	ldwImageGrid *anim3Image[eMaxFrames];

	ldwTextControl *playerCommField;
	ldwTextControl *animationName;
	ldwTextControl *currentSetLabel;
	ldwTextControl *headSetLabel;
	ldwTextControl *currentHeadSetLabel;
	ldwTextControl *animationEffectName;
	ldwTextControl *animationSpeedField;

	EAnimFrame     currentFrame;
	EHeadDirection currentHeadDirection;
	EDirection     currentDirection;
	EAnimEffect    currentAnimEffect;
	float		   currentAnimFps;

	CAnimControl animController;

};
#endif