//
//		Definition of the main game class
//
#ifndef THE_MAIN_SCENE_H
#define THE_MAIN_SCENE_H

#include "ldwScene.h"
#include "../World/WorldMap.h"
#include "ldwGameWindow.h"
#include "ldwImage.h"
#include "../Tools/DebuggerInterface.h"
#include "../Utils/theStringManager.h"
#include "../Villagers/Villager.h"
#include "../npc/theMiniButterflyClass.h"

class theRealtimeManager;
class theStringManager;

class ldwButton;
class ldwSoundTrack;
class ldwSoundFx;
class theGameState;
class ldwHintControl;
class ldwTextControl;

class theMainScene : public ldwScene, IDebugger
{
public:

	static theMainScene *Get();
	static theMainScene &Ref();
	static void Destroy();

	bool HandleKey(int messageId,int keyCode,int character);
	bool HandleMouse(int messageId,ldwPoint location);
	bool HandleMessage(int messageId,long data);

	const bool HandleKeyCharacter(const char cKey);
	const bool HandleKeyDown(const int iKeyCode);
	const bool HandleKeyUp(const int iKeyCode);
	const bool HandleMouseDown(const ldwPoint sMousePos);
	const bool HandleMouseUp(const ldwPoint sMousePos);
	const bool HandleMouseMove(const ldwPoint sMousePos);
	void HandleMausoleumButton();
	void AddHint(ldwHintControl *hint);
		
protected:

	void Activate(bool active);
	void Debug();
	void DrawScene();
	void DrawSceneFinal();
	void UpdateScene();

	void DrawWorld(const ldwRect& clipRect);
	void DrawUI();


	void HandleMenuButton();
	void HandleVillagerDetailsButton();
	void HandleTechButton();
	void HandleOverviewButton();
	void HandleCollectionButton();
	void HandleAchievementButton();
	void HandlePuzzlesButton();
	void HandleLastTipButton();
	

	void HandleMapClick(const ldwPoint sPos);
	bool HandleDropOnVillager(CVillager &Villager);
	void DropVillager();
	void SayCurrentContent(const ldwPoint sFeetPos);

	bool PtInFocusWindow(ldwPoint point);

private:

	typedef void (theMainScene::* LPBUTTONHANDLER)(void);

	//class enumerations
	enum EButton
	{
		eButton_None = -1,

		eButton_Menu,
		eButton_VillagerDetails,
		eButton_VillagerDetailsLeft,
		eButton_VillagerDetailsRight,
		eButton_Tech,
		eButton_Overview,
		eButton_Collection,
		eButton_Puzzles,
		eButton_LastTip,
		eButton_Achievement,

		eButton_NumberOf
	};

	//class data
	static theMainScene *	m_pInstance;

	//class methods
	//setup
	theMainScene();
	~theMainScene();

	ldwGameWindow *gameWindow;
	theGameState  *gs;
	CMiniButterflyClass *miniButterflies;
	theRealtimeManager *realtimeManager;
	theStringManager *stringMgr;

	ldwImage* 	  mFocusWindowBuffer;
	ldwImageGrid* mFocusWindowMask;

	ldwPoint m_sDragPos;

	ldwUINT32 secondTimer;

	bool m_draggingHut;
	bool m_mouseDownInPeepFocusWindow;
	bool m_oldSchoolFocusHandling;

	ldwPoint tutorialMouse;
	ldwPoint clickPoint;

	int tutorialCounter;

	int currentTipDebug;

	int cheatCode;

	bool showContent;
	//int clothingConfirmRequest;

	bool villagerClick;

	bool m_suspendAchievementPopup;

	bool dragging;
	int inHand;

	ldwRect blitRect;
	ldwRect displayRect;
	ldwRect schoolDrumRect;
	ldwRect tipRect;

	ldwPoint handPos;

	ldwImage *mainBG;
	ldwImage *mainBGTopBar;
	ldwImage *hand;
	ldwImage *handOpen;

	ldwButton  *lastTipButton;
	bool        mLastTipButtonShowing;

	ldwTextControl *popLabel;
	ldwTextControl *popField;
	ldwTextControl *foodLabel;
	ldwTextControl *foodField;
	ldwTextControl *techLabel;
	ldwTextControl *techField;
	ldwTextControl *nameLabel;
	ldwTextControl *nameField;
	ldwTextControl *skillLabel;
	ldwTextControl *skillField;
	ldwTextControl *actionLabel;
	ldwTextControl *actionField;
	ldwTextControl *faithLabel;
	ldwTextControl *playerCommField;
	ldwTextControl *numericManaLabel;
	ldwTextControl *numericCurrentManaField;
	ldwTextControl *numericMaxManaField;
	ldwTextControl *manaLabel;

};
#endif